Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 134085)
+++ channels/chan_sip.c	(working copy)
@@ -1474,6 +1474,7 @@
 struct sip_peer {
 	char name[80];			/*!< peer->name is the unique name of this object */
 	struct sip_socket socket;	/*!< Socket used for this peer */
+	unsigned int transpref:3; /*!< The prefered transport type (and the initial setting for an outbound call */
 	unsigned int transports:3; /*!< Transports (enum sip_transport) that are acceptable for this peer */
 	char secret[80];		/*!< Password */
 	char md5secret[80];		/*!< Password in MD5 */
@@ -2293,7 +2294,7 @@
 /*! \brief SIP TCP thread management function */
 static void *_sip_tcp_helper_thread(struct sip_pvt *pvt, struct ast_tcptls_session_instance *ser) 
 {
-	int res, cl;
+	int res, cl, bytesread;
 	struct sip_request req = { 0, } , reqcpy = { 0, };
 	struct sip_threadinfo *me;
 	char buf[1024] = "";
@@ -2358,14 +2359,16 @@
 		if (sscanf(get_header(&reqcpy, "Content-Length"), "%d", &cl)) {
 			while (cl > 0) {
 				ast_mutex_lock(&ser->lock);
-				if (!fread(buf, (cl < sizeof(buf)) ? cl : sizeof(buf), 1, ser->f)) {
+				bytesread = fread(buf, 1, (cl < sizeof(buf)) ? cl : sizeof(buf)-1, ser->f);
+				if (!bytesread) {
 					ast_mutex_unlock(&ser->lock);
 					goto cleanup;
 				}
+				buf[bytesread] = '\0';
 				ast_mutex_unlock(&ser->lock);
 				if (me->stop)
 					goto cleanup;
-				cl -= strlen(buf);
+				cl -= bytesread;
 				ast_str_append(&req.data, 0, "%s", buf);
 				req.len = req.data->used;
 			}
@@ -10852,7 +10855,7 @@
 					break;
 				}
 
-				if (peer->socket.type != req->socket.type ) {
+				if (peer->transpref != req->socket.type ) {
 					if (!(peer->transports & req->socket.type)) {
 						ast_log(LOG_ERROR,
 							"peer '%s' has contacted us over %s, but we only accept '%s' for this peer! ending call.\n",
@@ -10862,18 +10865,22 @@
 						ast_set_flag(&p->flags[0], SIP_PENDINGBYE);
 						transmit_response_with_date(p, "403 Forbidden", req);
 						res = AUTH_BAD_TRANSPORT;
-					} else if (peer->socket.type & SIP_TRANSPORT_TLS) {
-						ast_log(LOG_WARNING,
-							"peer '%s' HAS STOPPED USING TLS in favor of '%s' (but this was allowed in sip.conf)!\n",
-							peer->name, get_transport(req->socket.type)
-						);
 					} else {
-						ast_log(LOG_DEBUG,
-							"peer '%s' has contacted us over %s even though we prefer %s.\n", 
-							peer->name, get_transport(req->socket.type), get_transport(peer->socket.type)
-						);
+						if (peer->socket.type & SIP_TRANSPORT_TLS) {
+							ast_log(LOG_WARNING,
+								"peer '%s' HAS STOPPED USING TLS in favor of '%s' (but this was allowed in sip.conf)!\n",
+								peer->name, get_transport(req->socket.type)
+							);
+						} else {
+							ast_log(LOG_DEBUG,
+								"peer '%s' has contacted us over %s even though we prefer %s.\n", 
+								peer->name, get_transport(req->socket.type), get_transport(peer->transpref)
+							);
+						}
 					}
 				}
+
+				copy_socket_data(&peer->socket, &req->socket);
 			} 
 		}
 	}
@@ -19203,6 +19210,7 @@
 	}
 
 	ca.sin = *(sip_real_dst(p));
+	ca.sin.sin_family = AF_INET;
 
 	if ((ser = sip_tcp_locate(&ca.sin))) {
 		s->fd = ser->fd;
@@ -20712,8 +20720,10 @@
 				else
 					ast_log(LOG_NOTICE, "'%s' is not a valid transport type. if no other is specified, udp will be used.\n", trans);
 
-				if (!peer->socket.type) /*!< The first transport listed should be used for outgoing */
+				if (!peer->socket.type) { /*!< The first transport listed should be used for outgoing */
+					peer->transpref = peer->transports;
 					peer->socket.type = peer->transports;
+				}
 			}
 		} else if (realtime && !strcasecmp(v->name, "regseconds")) {
 			ast_get_time_t(v->value, &regseconds, 0, NULL);
